
"""%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

                Statement

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
"""

__intro__= "MLQD is a python package developed for Machine Learning-based Quantum Dissipative Dynamics,\n \t\t\t\t  Version 1.1.1\n\t\t\t https://github.com/Arif-PhyChem/MLQD" 
__author__ = "\n \t\t\t Copyright (c) 2023 Arif Ullah"
__copyright__ = "\nAll rights reserved. This work is licensed under the Apache Software License 2.0"
__mod__= "\n\nThe above copyright notice and this permission notice shall be included \nin all copies or substantial portions of the Software."
__disclaimer__="\n\nThe software is provided \"as is\" without warranty of any kind, express or implied, \nincluding but not limited to the warranties ofmerchantability, fitness for a particular \npurpose and noninfringement. In no event shall the authors or copyright holders be \nliable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, \narising from, out of or in connection with the software or the use or other dealings in the software."
__cite__="\n\n\t\t\t\t Cite as:\n\n1) Ullah A. and Dral P. O., New Journal of Physics, 2021, 23(11), 113019\n2) Ullah A. and Dral P. O., Nature Communications, 2022, 13(1), 1930\n3) Ullah A. and Dral P. O., Journal of Physical Chemistry Letters, 2022, 13(26), 6037\n4) Rodriguez L. E. H.; Ullah A.; Espinosa K. J. R.; Dral P. O. and A. A. Kananenka, Machine Learning: Science and Technology, 2022, 3(4), 045016\n5), Ullah A. and Dral P. O., 2023, arXiv.2303.01264"
__contributers__="\nContributers List:\n\n1) Arif Ullah (main) \n2) Pavlo O. Dral"
def statement():
    print('=================================================================')
    print(__intro__)
    print(__author__)
    print(__copyright__)
    print(__mod__)
    print(__disclaimer__)
    print(__cite__)
    print(__contributers__)
